@@ -25,6 +25,7 @@
 #include "dlg-ask-password.h"
 #include "file-utils.h"
 #include "fr-window.h"
+#include "gio-utils.h"
 #include "glib-utils.h"
 #include "gtk-utils.h"
 
@@ -58,7 +59,7 @@ destroy_cb (GtkWidget  *widget,
 
 
 static void
-ask_password__response_cb (GtkWidget  *dialog,
+ask_password__response_cb (GtkDialog  *dialog,
 			   int         response_id,
 			   DialogData *data)
 {
@@ -103,11 +104,7 @@ dlg_ask_password__common (FrWindow       *window,
 	char       *message;
 
 	data = g_new0 (DialogData, 1);
-	data->builder = _gtk_builder_new_from_resource ("ask-password.ui");
-	if (data->builder == NULL) {
-		g_free (data);
-		return;
-	}
+	data->builder = gtk_builder_new_from_resource (FILE_ROLLER_RESOURCE_UI_PATH "ask-password.ui");
 	data->window = window;
 	data->pwd_type = pwd_type;
 
@@ -127,7 +124,7 @@ dlg_ask_password__common (FrWindow       *window,
 	gtk_dialog_set_default_response (GTK_DIALOG (data->dialog),
 					 GTK_RESPONSE_OK);
 	gtk_style_context_add_class (gtk_widget_get_style_context (gtk_dialog_get_widget_for_response (GTK_DIALOG (data->dialog), GTK_RESPONSE_OK)),
-				     GTK_STYLE_CLASS_SUGGESTED_ACTION);
+				     "suggested-action");
 
 	data->password_entry = GET_WIDGET ("password_entry");
 
@@ -143,7 +140,7 @@ dlg_ask_password__common (FrWindow       *window,
 	} else
 		g_assert_not_reached ();
 
-	filename = _g_file_get_display_basename (file);
+	filename = _g_file_get_display_name (file);
 	/* Translators: %s is a filename */
 	message = g_strdup_printf (_("Password required for “%s”"), filename);
 	gtk_label_set_label (GTK_LABEL (GET_WIDGET ("title_label")), message);
@@ -166,11 +163,11 @@ dlg_ask_password__common (FrWindow       *window,
 
 	/* Set the signals handlers. */
 
-	g_signal_connect (G_OBJECT (data->dialog),
+	g_signal_connect (GTK_DIALOG (data->dialog),
 			  "destroy",
 			  G_CALLBACK (destroy_cb),
 			  data);
-	g_signal_connect (G_OBJECT (data->dialog),
+	g_signal_connect (GTK_DIALOG (data->dialog),
 			  "response",
 			  G_CALLBACK (ask_password__response_cb),
 			  data);
