@@ -22,6 +22,7 @@
 #include <config.h>
 #include <string.h>
 #include <gtk/gtk.h>
+#include "gio-utils.h"
 #include "glib-utils.h"
 #include "file-utils.h"
 #include "gtk-utils.h"
@@ -63,11 +64,7 @@ dlg_prop (FrWindow *window)
 
 	data = g_new (DialogData, 1);
 
-	data->builder = _gtk_builder_new_from_resource ("properties.ui");
-	if (data->builder == NULL) {
-		g_free (data);
-		return;
-	}
+	data->builder = gtk_builder_new_from_resource (FILE_ROLLER_RESOURCE_UI_PATH "properties.ui");
 
 	/* Get the widgets. */
 	table = _gtk_builder_get_widget (data->builder, "content");
@@ -101,7 +98,7 @@ dlg_prop (FrWindow *window)
 	/**/
 
 	label = _gtk_builder_get_widget (data->builder, "p_name_label");
-	utf8_name = _g_file_get_display_basename (fr_window_get_archive_file (window));
+	utf8_name = _g_file_get_display_name (fr_window_get_archive_file (window));
 	gtk_label_set_text (GTK_LABEL (label), utf8_name);
 
 	title_txt = g_strdup_printf (_("%s Properties"), utf8_name);
@@ -117,16 +114,21 @@ dlg_prop (FrWindow *window)
 
 	/**/
 
-	label = _gtk_builder_get_widget (data->builder, "p_date_label");
-	s = _g_time_to_string (_g_file_get_file_mtime (fr_window_get_archive_file (window)));
-	gtk_label_set_text (GTK_LABEL (label), s);
-	g_free (s);
+	{
+		g_autoptr (GDateTime) date_time;
+
+		label = _gtk_builder_get_widget (data->builder, "p_date_label");
+		date_time = g_date_time_new_from_unix_local (_g_file_get_file_mtime (fr_window_get_archive_file (window)));
+		s = g_date_time_format (date_time, _("%d %B %Y, %H:%M"));
+		gtk_label_set_text (GTK_LABEL (label), s);
+		g_free (s);
+	}
 
 	/**/
 
 	label = _gtk_builder_get_widget (data->builder, "p_size_label");
 	size = _g_file_get_file_size (fr_window_get_archive_file (window));
-	s = g_format_size (size);
+	s = g_format_size_full (size, G_FORMAT_SIZE_LONG_FORMAT);
 	gtk_label_set_text (GTK_LABEL (label), s);
 	g_free (s);
 
@@ -134,16 +136,14 @@ dlg_prop (FrWindow *window)
 
 	uncompressed_size = 0;
 	if (fr_window_archive_is_present (window)) {
-		int i;
-
-		for (i = 0; i < window->archive->files->len; i++) {
-			FileData *fd = g_ptr_array_index (window->archive->files, i);
+		for (guint i = 0; i < window->archive->files->len; i++) {
+			FrFileData *fd = g_ptr_array_index (window->archive->files, i);
 			uncompressed_size += fd->size;
 		}
 	}
 
 	label = _gtk_builder_get_widget (data->builder, "p_uncomp_size_label");
-	s = g_format_size (uncompressed_size);
+	s = g_format_size_full (uncompressed_size, G_FORMAT_SIZE_LONG_FORMAT);
 	gtk_label_set_text (GTK_LABEL (label), s);
 	g_free (s);
 
@@ -168,7 +168,7 @@ dlg_prop (FrWindow *window)
 
 	/* Set the signals handlers. */
 
-	g_signal_connect (G_OBJECT (data->dialog),
+	g_signal_connect (data->dialog,
 			  "destroy",
 			  G_CALLBACK (destroy_cb),
 			  data);
