@@ -29,7 +29,7 @@
 #include <gio/gio.h>
 #include <archive.h>
 #include <archive_entry.h>
-#include "file-data.h"
+#include "fr-file-data.h"
 #include "file-utils.h"
 #include "fr-error.h"
 #include "fr-archive-libarchive.h"
@@ -42,13 +42,45 @@
 #define FILE_ATTRIBUTES_NEEDED_BY_ARCHIVE_ENTRY ("standard::*,time::*,access::*,unix::*")
 
 
-struct _FrArchiveLibarchivePrivate {
+/* workaround the struct types of libarchive */
+typedef struct archive _archive_read_ctx;
+
+static void
+_archive_read_ctx_free(_archive_read_ctx *arch)
+{
+	archive_read_free(arch);
+}
+
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(_archive_read_ctx, _archive_read_ctx_free)
+
+typedef struct archive _archive_write_ctx;
+
+static void
+_archive_write_ctx_free(_archive_write_ctx *arch)
+{
+	archive_write_free(arch);
+}
+
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(_archive_write_ctx, _archive_write_ctx_free)
+
+typedef struct archive_entry _archive_entry_ctx;
+
+static void
+_archive_entry_ctx_free(_archive_entry_ctx *entry)
+{
+	archive_entry_free(entry);
+}
+
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(_archive_entry_ctx, _archive_entry_ctx_free)
+
+
+typedef struct {
 	gssize compressed_size;
 	gssize uncompressed_size;
-};
+} FrArchiveLibarchivePrivate;
 
 
-G_DEFINE_TYPE_WITH_PRIVATE (FrArchiveLibarchive, fr_archive_libarchive, FR_TYPE_ARCHIVE)
+G_DEFINE_FINAL_TYPE_WITH_PRIVATE (FrArchiveLibarchive, fr_archive_libarchive, FR_TYPE_ARCHIVE)
 
 
 static void
@@ -68,17 +100,14 @@ fr_archive_libarchive_finalize (GObject *object)
 
 const char *libarchiver_mime_types[] = {
 	"application/epub+zip",
-	"application/vnd.debian.binary-package",
 	"application/vnd.ms-cab-compressed",
 	"application/x-7z-compressed",
-	"application/x-ar",
 	"application/x-bzip-compressed-tar",
 	"application/x-cbr",
 	"application/x-cbz",
 	"application/x-cd-image",
 	"application/x-compressed-tar",
 	"application/x-cpio",
-	"application/x-deb",
 	"application/x-lha",
 	"application/x-lrzip-compressed-tar",
 	"application/x-lzip-compressed-tar",
@@ -105,20 +134,22 @@ fr_archive_libarchive_get_mime_types (FrArchive *archive)
 }
 
 
-static FrArchiveCap
+static FrArchiveCaps
 fr_archive_libarchive_get_capabilities (FrArchive  *archive,
 					const char *mime_type,
 					gboolean    check_command)
 {
-	FrArchiveCap capabilities;
+	FrArchiveCaps capabilities;
 
 	capabilities = FR_ARCHIVE_CAN_STORE_MANY_FILES;
 
 	/* give priority to 7z* for 7z archives. */
 	if (strcmp (mime_type, "application/x-7z-compressed") == 0) {
-		if (_g_program_is_available ("7za", check_command)
-		    || _g_program_is_available ("7zr", check_command)
-		    || _g_program_is_available ("7z", check_command))
+		if (_g_program_is_available ("7zz", TRUE)
+		    || _g_program_is_available ("7zzs", TRUE)
+		    || _g_program_is_available ("7za", TRUE)
+		    || _g_program_is_available ("7zr", TRUE)
+		    || _g_program_is_available ("7z", TRUE))
 		{
 			return capabilities;
 		}
@@ -126,7 +157,7 @@ fr_archive_libarchive_get_capabilities (FrArchive  *archive,
 
 	/* give priority to 7za that supports CAB files better. */
 	if ((strcmp (mime_type, "application/vnd.ms-cab-compressed") == 0)
-	    && _g_program_is_available ("7za", check_command))
+	    && _g_program_is_available ("7za", TRUE))
 	{
 		return capabilities;
 	}
@@ -135,13 +166,15 @@ fr_archive_libarchive_get_capabilities (FrArchive  *archive,
 	if ((strcmp (mime_type, "application/zip") == 0)
 	    || (strcmp (mime_type, "application/x-cbz") == 0))
 	{
-		if (_g_program_is_available ("7z", check_command)) {
+		if (_g_program_is_available ("7zz", TRUE)
+		    || _g_program_is_available ("7zzs", TRUE)
+		    || _g_program_is_available ("7z", TRUE)) {
 			return capabilities;
 		}
-		if (!_g_program_is_available ("unzip", check_command)) {
+		if (!_g_program_is_available ("unzip", TRUE)) {
 			capabilities |= FR_ARCHIVE_CAN_READ;
 		}
-		if (!_g_program_is_available ("zip", check_command)) {
+		if (!_g_program_is_available ("zip", TRUE)) {
 			capabilities |= FR_ARCHIVE_CAN_WRITE;
 		}
 		return capabilities;
@@ -151,9 +184,9 @@ fr_archive_libarchive_get_capabilities (FrArchive  *archive,
 	if ((strcmp (mime_type, "application/x-rar") == 0)
 	    || (strcmp (mime_type, "application/x-cbr") == 0))
 	{
-		if (_g_program_is_available ("rar", check_command)
-		    || _g_program_is_available ("unrar", check_command)
-		    || _g_program_is_available ("unar", check_command)) {
+		if (_g_program_is_available ("rar", TRUE)
+		    || _g_program_is_available ("unrar", TRUE)
+		    || _g_program_is_available ("unar", TRUE)) {
 			return capabilities;
 		}
 	}
@@ -169,8 +202,6 @@ fr_archive_libarchive_get_capabilities (FrArchive  *archive,
 	/* read-only formats */
 	if ((strcmp (mime_type, "application/vnd.ms-cab-compressed") == 0)
 	    || (strcmp (mime_type, "application/x-cbr") == 0)
-	    || (strcmp (mime_type, "application/x-deb") == 0)
-	    || (strcmp (mime_type, "application/vnd.debian.binary-package") == 0)
 	    || (strcmp (mime_type, "application/x-lha") == 0)
 	    || (strcmp (mime_type, "application/x-rar") == 0)
 	    || (strcmp (mime_type, "application/x-rpm") == 0)
@@ -230,6 +261,8 @@ load_data_free (LoadData *load_data)
 	g_free (load_data);
 }
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (LoadData, load_data_free)
+
 
 static int
 load_data_open (struct archive *a,
@@ -241,8 +274,9 @@ load_data_open (struct archive *a,
 		return ARCHIVE_FATAL;
 
 	if (g_simple_async_result_get_source_tag (load_data->result) == fr_archive_list) {
-		FR_ARCHIVE_LIBARCHIVE (load_data->archive)->priv->compressed_size = 0;
-		FR_ARCHIVE_LIBARCHIVE (load_data->archive)->priv->uncompressed_size = 0;
+		FrArchiveLibarchivePrivate *private = fr_archive_libarchive_get_instance_private (FR_ARCHIVE_LIBARCHIVE (load_data->archive));
+		private->compressed_size = 0;
+		private->uncompressed_size = 0;
 	}
 
 	load_data->istream = (GInputStream *) g_file_read (fr_archive_get_file (load_data->archive),
@@ -272,9 +306,10 @@ load_data_read (struct archive  *a,
 
 	/* update the progress only if listing the content */
 	if (g_simple_async_result_get_source_tag (load_data->result) == fr_archive_list) {
+		FrArchiveLibarchivePrivate *private = fr_archive_libarchive_get_instance_private (FR_ARCHIVE_LIBARCHIVE (load_data->archive));
 		fr_archive_progress_set_completed_bytes (load_data->archive,
 							 g_seekable_tell (G_SEEKABLE (load_data->istream)));
-		FR_ARCHIVE_LIBARCHIVE (load_data->archive)->priv->compressed_size += bytes;
+		private->compressed_size += bytes;
 	}
 
 	return bytes;
@@ -368,7 +403,7 @@ load_data_close (struct archive *a,
 
 static int
 create_read_object (LoadData        *load_data,
-		    struct archive **a)
+                    _archive_read_ctx **a)
 {
 	*a = archive_read_new ();
 	archive_read_support_filter_all (*a);
@@ -392,7 +427,7 @@ static goffset
 _g_file_get_size (GFile        *file,
 		  GCancellable *cancellable)
 {
-	GFileInfo *info;
+	g_autoptr (GFileInfo) info = NULL;
 	goffset    size;
 
 	info = g_file_query_info (file,
@@ -404,7 +439,6 @@ _g_file_get_size (GFile        *file,
 		return 0;
 
 	size = g_file_info_get_size (info);
-	g_object_unref (info);
 
 	return size;
 }
@@ -413,7 +447,7 @@ _g_file_get_size (GFile        *file,
 static GError *
 _g_error_new_from_archive_error (const char *s)
 {
-	char   *msg;
+	g_autofree char *msg = NULL;
 	GError *error;
 
 	msg = (s != NULL) ? g_locale_to_utf8 (s, -1, NULL, NULL, NULL) : NULL;
@@ -421,8 +455,6 @@ _g_error_new_from_archive_error (const char *s)
 		msg = g_strdup ("Fatal error");
 	error = g_error_new_literal (FR_ERROR, FR_ERROR_COMMAND_ERROR, msg);
 
-	g_free (msg);
-
 	return error;
 }
 
@@ -432,8 +464,8 @@ list_archive_thread (GSimpleAsyncResult *result,
 		     GObject            *object,
 		     GCancellable       *cancellable)
 {
-	LoadData             *load_data;
-	struct archive       *a;
+	g_autoptr (LoadData) load_data = NULL;
+	g_autoptr (_archive_read_ctx) a = NULL;
 	struct archive_entry *entry;
 	int                   r;
 
@@ -444,22 +476,22 @@ list_archive_thread (GSimpleAsyncResult *result,
 
 	r = create_read_object (load_data, &a);
 	if (r != ARCHIVE_OK) {
-		archive_read_free(a);
 		return;
 	}
 
 	while ((r = archive_read_next_header (a, &entry)) == ARCHIVE_OK) {
-		FileData   *file_data;
+		FrFileData *file_data;
 		const char *pathname;
 
 		if (g_cancellable_is_cancelled (cancellable))
 			break;
 
-		file_data = file_data_new ();
+		file_data = fr_file_data_new ();
 
 		if (archive_entry_size_is_set (entry)) {
+			FrArchiveLibarchivePrivate *private = fr_archive_libarchive_get_instance_private (FR_ARCHIVE_LIBARCHIVE (load_data->archive));
 			file_data->size = archive_entry_size (entry);
-			FR_ARCHIVE_LIBARCHIVE (load_data->archive)->priv->uncompressed_size += file_data->size;
+			private->uncompressed_size += file_data->size;
 		}
 
 		if (archive_entry_mtime_is_set (entry))
@@ -505,9 +537,6 @@ list_archive_thread (GSimpleAsyncResult *result,
 		g_cancellable_set_error_if_cancelled (cancellable, &load_data->error);
 	if (load_data->error != NULL)
 		g_simple_async_result_set_from_error (result, load_data->error);
-
-	archive_read_free (a);
-	load_data_free (load_data);
 }
 
 
@@ -573,6 +602,8 @@ extract_data_free (ExtractData *extract_data)
 	load_data_free (LOAD_DATA (extract_data));
 }
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (ExtractData, extract_data_free)
+
 
 static gboolean
 extract_data_get_extraction_requested (ExtractData *extract_data,
@@ -703,7 +734,7 @@ _g_file_contains_symlinks_in_path (const char *relative_path,
 				   GHashTable *symlinks)
 {
 	gboolean  contains_symlinks = FALSE;
-	GFile    *parent;
+	g_autoptr (GFile) parent = NULL;
 	char    **components;
 	int       i;
 
@@ -722,7 +753,6 @@ _g_file_contains_symlinks_in_path (const char *relative_path,
 			continue;
 
 		tmp = g_file_get_child (parent, components[i]);
-		g_object_unref (parent);
 		parent = tmp;
 
 		if (g_hash_table_contains (symlinks, parent)) {
@@ -732,7 +762,6 @@ _g_file_contains_symlinks_in_path (const char *relative_path,
 	}
 
 	g_strfreev (components);
-	g_object_unref (parent);
 
 	return contains_symlinks;
 }
@@ -743,13 +772,13 @@ extract_archive_thread (GSimpleAsyncResult *result,
 			GObject            *object,
 			GCancellable       *cancellable)
 {
-	ExtractData          *extract_data;
+	g_autoptr (ExtractData) extract_data = NULL;
 	LoadData             *load_data;
-	GHashTable           *checked_folders;
-	GHashTable           *created_files;
-	GHashTable           *folders_created_during_extraction;
-	GHashTable           *symlinks;
-	struct archive       *a;
+	g_autoptr (GHashTable) checked_folders = NULL;
+	g_autoptr (GHashTable) created_files = NULL;
+	g_autoptr (GHashTable) folders_created_during_extraction = NULL;
+	g_autoptr (GHashTable) symlinks = NULL;
+	g_autoptr (_archive_read_ctx) a = NULL;
 	struct archive_entry *entry;
 	int                   r;
 
@@ -758,7 +787,6 @@ extract_archive_thread (GSimpleAsyncResult *result,
 
 	r = create_read_object (load_data, &a);
 	if (r != ARCHIVE_OK) {
-		archive_read_free(a);
 		return;
 	}
 
@@ -770,14 +798,15 @@ extract_archive_thread (GSimpleAsyncResult *result,
 
 	while ((r = archive_read_next_header (a, &entry)) == ARCHIVE_OK) {
 		const char    *pathname;
-		char          *fullpath;
+		g_autofree char * fullpath = NULL;
 		const char    *relative_path;
-		GFile         *file;
-		GFile         *parent;
-		GOutputStream *ostream;
+		g_autoptr (GFile) file = NULL;
+		g_autoptr (GFile) parent = NULL;
+		g_autoptr (GOutputStream) ostream = NULL;
 		const void    *buffer;
 		size_t         buffer_size;
-		int64_t        target_offset, actual_offset;
+		int64_t target_offset = 0;
+		int64_t actual_offset = 0;
 		GError        *local_error = NULL;
 		__LA_MODE_T    filetype;
 
@@ -820,7 +849,7 @@ extract_archive_thread (GSimpleAsyncResult *result,
 		if ((g_hash_table_lookup (folders_created_during_extraction, file) == NULL)
 		    && (extract_data->skip_older || ! extract_data->overwrite))
 		{
-			GFileInfo *info;
+			g_autoptr (GFileInfo) info = NULL;
 
 			info = g_file_query_info (file,
 						  G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME "," G_FILE_ATTRIBUTE_TIME_MODIFIED,
@@ -841,11 +870,7 @@ extract_archive_thread (GSimpleAsyncResult *result,
 						skip = TRUE;
 				}
 
-				g_object_unref (info);
-
 				if (skip) {
-					g_object_unref (file);
-
 					archive_read_data_skip (a);
 					fr_archive_progress_inc_completed_bytes (load_data->archive, archive_entry_size_is_set (entry) ? archive_entry_size (entry) : 0);
 
@@ -860,7 +885,6 @@ extract_archive_thread (GSimpleAsyncResult *result,
 			else {
 				if (! g_error_matches (local_error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND)) {
 					load_data->error = local_error;
-					g_object_unref (info);
 					break;
 				}
 				g_clear_error (&local_error);
@@ -899,7 +923,6 @@ extract_archive_thread (GSimpleAsyncResult *result,
 				}
 			}
 		}
-		g_object_unref (parent);
 
 		/* create the file */
 
@@ -910,17 +933,16 @@ extract_archive_thread (GSimpleAsyncResult *result,
 
 			linkname = archive_entry_hardlink (entry);
 			if (linkname != NULL) {
-				char        *link_fullpath;
-				const char  *relative_path;
-				GFile       *link_file;
-				char        *oldname;
-				char        *newname;
+				g_autofree char *link_fullpath = NULL;
+				const char *relative_path;
+				g_autoptr (GFile) link_file = NULL;
+				g_autofree char *oldname = NULL;
+				g_autofree char *newname = NULL;
 				int          r;
 
 				link_fullpath = (*linkname == '/') ? g_strdup (linkname) : g_strconcat ("/", linkname, NULL);
 				relative_path = _g_path_get_relative_basename_safe (link_fullpath, extract_data->base_dir, extract_data->junk_paths);
 				if (relative_path == NULL) {
-					g_free (link_fullpath);
 					archive_read_data_skip (a);
 					continue;
 				}
@@ -946,21 +968,10 @@ extract_archive_thread (GSimpleAsyncResult *result,
 						filetype = AE_IFREG; /* treat as a regular file to save the data */
 				}
 				else {
-					char *uri;
-					char *msg;
-
-					uri = g_file_get_uri (file);
-					msg = g_strdup_printf ("Could not create the hard link %s", uri);
+					g_autofree char *uri = g_file_get_uri (file);
+					g_autofree char *msg = g_strdup_printf ("Could not create the hard link %s", uri);
 					load_data->error = g_error_new_literal (G_IO_ERROR, G_IO_ERROR_FAILED, msg);
-
-					g_free (msg);
-					g_free (uri);
 				}
-
-				g_free (newname);
-				g_free (oldname);
-				g_object_unref (link_file);
-				g_free (link_fullpath);
 			}
 		}
 
@@ -982,7 +993,6 @@ extract_archive_thread (GSimpleAsyncResult *result,
 				if (ostream == NULL)
 					break;
 
-				actual_offset = 0;
 				while ((r = archive_read_data_block (a, &buffer, &buffer_size, &target_offset)) == ARCHIVE_OK) {
 					gsize bytes_written;
 
@@ -1003,8 +1013,6 @@ extract_archive_thread (GSimpleAsyncResult *result,
 				if ((r == ARCHIVE_EOF) && (target_offset > actual_offset))
 					_g_output_stream_add_padding (extract_data, ostream, target_offset, actual_offset, cancellable, &load_data->error);
 
-				_g_object_unref (ostream);
-
 				if (r != ARCHIVE_EOF)
 					load_data->error = _g_error_new_from_archive_error (archive_error_string (a));
 				else
@@ -1038,9 +1046,6 @@ extract_archive_thread (GSimpleAsyncResult *result,
 			}
 		}
 
-		g_object_unref (file);
-		g_free (fullpath);
-
 		if (load_data->error != NULL)
 			break;
 
@@ -1059,13 +1064,6 @@ extract_archive_thread (GSimpleAsyncResult *result,
 		g_cancellable_set_error_if_cancelled (cancellable, &load_data->error);
 	if (load_data->error != NULL)
 		g_simple_async_result_set_from_error (result, load_data->error);
-
-	g_hash_table_unref (folders_created_during_extraction);
-	g_hash_table_unref (created_files);
-	g_hash_table_unref (checked_folders);
-	g_hash_table_unref (symlinks);
-	archive_read_free (a);
-	extract_data_free (extract_data);
 }
 
 
@@ -1219,6 +1217,8 @@ save_data_free (SaveData *save_data)
 	load_data_free (LOAD_DATA (save_data));
 }
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (SaveData, save_data_free)
+
 
 static int
 save_data_open (struct archive *a,
@@ -1226,9 +1226,9 @@ save_data_open (struct archive *a,
 {
 	SaveData *save_data = client_data;
 	LoadData *load_data = LOAD_DATA (save_data);
-	GFile    *parent;
-	char     *basename;
-	char     *tmpname;
+	g_autoptr (GFile) parent = NULL;
+	g_autofree char * basename = NULL;
+	g_autofree char * tmpname = NULL;
 
 	if (load_data->error != NULL)
 		return ARCHIVE_FATAL;
@@ -1239,10 +1239,6 @@ save_data_open (struct archive *a,
 	save_data->tmp_file = g_file_get_child (parent, tmpname);
 	save_data->ostream = (GOutputStream *) g_file_create (save_data->tmp_file, G_FILE_CREATE_NONE, load_data->cancellable, &load_data->error);
 
-	g_free (tmpname);
-	g_free (basename);
-	g_object_unref (parent);
-
 	return (save_data->ostream != NULL) ? ARCHIVE_OK : ARCHIVE_FATAL;
 }
 
@@ -1358,9 +1354,6 @@ _archive_write_set_format_from_context (struct archive *a,
 	else if (_g_str_equal (mime_type, "application/x-xar")) {
 		archive_write_set_format_xar (a);
 	}
-	else if (_g_str_equal (mime_type, "application/x-ar")) {
-		archive_write_set_format_ar_svr4 (a);
-	}
 	else if (_g_str_equal (mime_type, "application/x-7z-compressed")) {
 		archive_write_set_format_7zip (a);
 	}
@@ -1448,6 +1441,12 @@ _archive_write_set_format_from_context (struct archive *a,
 		}
 		if (compression_level != NULL)
 			archive_write_set_filter_option (a, NULL, "compression-level", compression_level);
+
+		/* set the amount of threads */
+
+		if (archive_filter == ARCHIVE_FILTER_XZ) {
+			archive_write_set_filter_option (a, NULL, "threads", fr_get_thread_count());
+                }
 	}
 }
 
@@ -1555,8 +1554,8 @@ _archive_write_file (struct archive       *b,
 		     GCancellable         *cancellable)
 {
 	LoadData             *load_data = LOAD_DATA (save_data);
-	GFileInfo            *info;
-	struct archive_entry *w_entry;
+	g_autoptr (GFileInfo) info = NULL;
+	g_autoptr (_archive_entry_ctx) w_entry = NULL;
 	int                   rb;
 
 	/* write the file header */
@@ -1571,16 +1570,12 @@ _archive_write_file (struct archive       *b,
 
 	w_entry = archive_entry_new ();
 	if (! _archive_entry_copy_file_info (w_entry, info, save_data)) {
-		archive_entry_free (w_entry);
-		g_object_unref (info);
 		return WRITE_ACTION_SKIP_ENTRY;
 	}
 
 	/* honor the update flag */
 
 	if (save_data->update && (r_entry != NULL) && (archive_entry_mtime (w_entry) < archive_entry_mtime (r_entry))) {
-		archive_entry_free (w_entry);
-		g_object_unref (info);
 		return WRITE_ACTION_WRITE_ENTRY;
 	}
 
@@ -1590,7 +1585,7 @@ _archive_write_file (struct archive       *b,
 	/* write the file data */
 
 	if (g_file_info_get_file_type (info) == G_FILE_TYPE_REGULAR) {
-		GInputStream *istream;
+		g_autoptr (GInputStream) istream = NULL;
 
 		istream = (GInputStream *) g_file_read (add_file->file, cancellable, &load_data->error);
 		if (istream != NULL) {
@@ -1600,8 +1595,6 @@ _archive_write_file (struct archive       *b,
 				archive_write_data (b, save_data->buffer, bytes_read);
 				fr_archive_progress_inc_completed_bytes (load_data->archive, bytes_read);
 			}
-
-			g_object_unref (istream);
 		}
 	}
 
@@ -1610,9 +1603,6 @@ _archive_write_file (struct archive       *b,
 	if ((load_data->error == NULL) && (rb <= ARCHIVE_FAILED))
 		load_data->error = g_error_new_literal (FR_ERROR, FR_ERROR_COMMAND_ERROR, archive_error_string (b));
 
-	archive_entry_free (w_entry);
-	g_object_unref (info);
-
 	return (load_data->error == NULL) ? WRITE_ACTION_SKIP_ENTRY : WRITE_ACTION_ABORT;
 }
 
@@ -1622,9 +1612,10 @@ save_archive_thread (GSimpleAsyncResult *result,
 		     GObject            *object,
 		     GCancellable       *cancellable)
 {
-	SaveData             *save_data;
+	g_autoptr (SaveData) save_data = NULL;
 	LoadData             *load_data;
-	struct archive       *a, *b;
+	g_autoptr (_archive_read_ctx) a = NULL;
+	g_autoptr (_archive_write_ctx) b = NULL;
 	struct archive_entry *r_entry;
 	int                   ra = ARCHIVE_OK, rb = ARCHIVE_OK;
 
@@ -1642,7 +1633,7 @@ save_archive_thread (GSimpleAsyncResult *result,
 		save_data->begin_operation (save_data, save_data->user_data);
 
 	while ((load_data->error == NULL) && (ra = archive_read_next_header (a, &r_entry)) == ARCHIVE_OK) {
-		struct archive_entry *w_entry;
+		g_autoptr (_archive_entry_ctx) w_entry = NULL;
 		WriteAction           action;
 
 		if (g_cancellable_is_cancelled (cancellable))
@@ -1685,8 +1676,6 @@ save_archive_thread (GSimpleAsyncResult *result,
 		}
 		else if (action == WRITE_ACTION_SKIP_ENTRY)
 			fr_archive_progress_inc_completed_bytes (load_data->archive, archive_entry_size (r_entry));
-
-		archive_entry_free (w_entry);
 	}
 
 	if (g_error_matches (load_data->error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
@@ -1705,10 +1694,6 @@ save_archive_thread (GSimpleAsyncResult *result,
 		g_cancellable_set_error_if_cancelled (cancellable, &load_data->error);
 	if (load_data->error != NULL)
 		g_simple_async_result_set_from_error (result, load_data->error);
-
-	archive_read_free (a);
-	archive_write_free (b);
-	save_data_free (save_data);
 }
 
 
@@ -1799,7 +1784,7 @@ _add_files_begin (SaveData *save_data,
 	fr_archive_progress_set_total_files (load_data->archive, add_data->n_files_to_add);
 
 	if (load_data->archive->files_to_add_size == 0) {
-		GList *files_to_add;
+		g_autoptr (GList) files_to_add = NULL;
 		GList *scan;
 
 		files_to_add = g_hash_table_get_values (add_data->files_to_add);
@@ -1811,12 +1796,10 @@ _add_files_begin (SaveData *save_data,
 
 			load_data->archive->files_to_add_size += _g_file_get_size (add_file->file, load_data->cancellable);
 		}
-
-		g_list_free (files_to_add);
 	}
 
-	fr_archive_progress_set_total_bytes (load_data->archive,
-			FR_ARCHIVE_LIBARCHIVE (load_data->archive)->priv->uncompressed_size + load_data->archive->files_to_add_size);
+	FrArchiveLibarchivePrivate *private = fr_archive_libarchive_get_instance_private (FR_ARCHIVE_LIBARCHIVE (load_data->archive));
+	fr_archive_progress_set_total_bytes (load_data->archive, private->uncompressed_size + load_data->archive->files_to_add_size);
 }
 
 
@@ -1856,7 +1839,7 @@ _add_files_end (SaveData *save_data,
 {
 	AddData  *add_data = user_data;
 	LoadData *load_data = LOAD_DATA (save_data);
-	GList    *remaining_files;
+	g_autoptr (GList) remaining_files = NULL;
 	GList    *scan;
 
 	/* allow to add files to a new archive */
@@ -1885,8 +1868,6 @@ _add_files_end (SaveData *save_data,
 
 		fr_archive_progress_inc_completed_files (load_data->archive, 1);
 	}
-
-	g_list_free (remaining_files);
 }
 
 
@@ -1920,18 +1901,12 @@ fr_archive_libarchive_add_files (FrArchive           *archive,
 
 	for (scan = file_list; scan; scan = scan->next) {
 		GFile *file = G_FILE (scan->data);
-		char  *relative_pathname;
-		char  *archive_pathname;
-
-		relative_pathname = g_file_get_relative_path (base_dir, file);
-		archive_pathname = g_build_filename (dest_dir, relative_pathname, NULL);
+		g_autofree char *relative_pathname = g_file_get_relative_path (base_dir, file);
+		g_autofree char *archive_pathname = g_build_filename (dest_dir, relative_pathname, NULL);
 		g_hash_table_insert (add_data->files_to_add,
 				     g_strdup (archive_pathname),
 				     add_file_new (file, archive_pathname));
 		add_data->n_files_to_add++;
-
-		g_free (archive_pathname);
-		g_free (relative_pathname);
 	}
 
 	_fr_archive_libarchive_save (archive,
@@ -1978,10 +1953,10 @@ _remove_files_begin (SaveData *save_data,
 {
 	LoadData   *load_data = LOAD_DATA (save_data);
 	RemoveData *remove_data = user_data;
+	FrArchiveLibarchivePrivate *private = fr_archive_libarchive_get_instance_private (FR_ARCHIVE_LIBARCHIVE (load_data->archive));
 
 	fr_archive_progress_set_total_files (load_data->archive, remove_data->n_files_to_remove);
-	fr_archive_progress_set_total_bytes (load_data->archive,
-					     FR_ARCHIVE_LIBARCHIVE (load_data->archive)->priv->uncompressed_size);
+	fr_archive_progress_set_total_bytes (load_data->archive, private->uncompressed_size);
 }
 
 
@@ -2079,8 +2054,8 @@ _rename_files_begin (SaveData *save_data,
 	RenameData *rename_data = user_data;
 
 	fr_archive_progress_set_total_files (load_data->archive, rename_data->n_files_to_rename);
-	fr_archive_progress_set_total_bytes (load_data->archive,
-				FR_ARCHIVE_LIBARCHIVE (load_data->archive)->priv->uncompressed_size);
+	FrArchiveLibarchivePrivate *private = fr_archive_libarchive_get_instance_private (FR_ARCHIVE_LIBARCHIVE (load_data->archive));
+	fr_archive_progress_set_total_bytes (load_data->archive, private->uncompressed_size);
 }
 
 
@@ -2129,26 +2104,17 @@ fr_archive_libarchive_rename (FrArchive           *archive,
 	rename_data->n_files_to_rename = 0;
 
 	if (is_dir) {
-		char  *old_dirname;
-		char  *new_dirname;
-		int    old_dirname_len;
+		g_autofree char *old_dirname = g_build_filename (current_dir + 1, old_name, "/", NULL);
+		int old_dirname_len = strlen (old_dirname);
+		g_autofree char *new_dirname = g_build_filename (current_dir + 1, new_name, "/", NULL);
 		GList *scan;
 
-		old_dirname = g_build_filename (current_dir + 1, old_name, "/", NULL);
-		old_dirname_len = strlen (old_dirname);
-		new_dirname = g_build_filename (current_dir + 1, new_name, "/", NULL);
-
 		for (scan = file_list; scan; scan = scan->next) {
 			char *old_pathname = scan->data;
-			char *new_pathname;
-
-			new_pathname = g_build_filename (new_dirname, old_pathname + old_dirname_len, NULL);
+			char *new_pathname = g_build_filename (new_dirname, old_pathname + old_dirname_len, NULL);
 			g_hash_table_insert (rename_data->files_to_rename, g_strdup (old_pathname), new_pathname);
 			rename_data->n_files_to_rename++;
 		}
-
-		g_free (new_dirname);
-		g_free (old_dirname);
 	}
 	else {
 		char *old_pathname = (char *) file_list->data;
@@ -2210,14 +2176,12 @@ fr_archive_libarchive_paste_clipboard (FrArchive           *archive,
 	for (scan = files; scan; scan = scan->next) {
 		const char *old_name = (char *) scan->data;
 		char       *new_name;
-		GFile      *file;
+		g_autoptr (GFile) file = NULL;
 
 		new_name = g_build_filename (current_dir, old_name + strlen (base_dir) - 1, NULL);
 		file = _g_file_append_path (tmp_dir, old_name, NULL);
 		g_hash_table_insert (add_data->files_to_add, new_name, add_file_new (file, new_name));
 		add_data->n_files_to_add++;
-
-		g_object_unref (file);
 	}
 
 	_fr_archive_libarchive_save (archive,
@@ -2264,17 +2228,12 @@ fr_archive_libarchive_add_dropped_files (FrArchive           *archive,
 
 	for (scan = file_list; scan; scan = scan->next) {
 		GFile *file = G_FILE (scan->data);
-		char  *basename;
-		char  *archive_pathname;
+		g_autofree char *basename = g_file_get_basename (file);
+		g_autofree char *archive_pathname = g_build_filename (dest_dir, basename, NULL);
 
-		basename = g_file_get_basename (file);
-		archive_pathname = g_build_filename (dest_dir, basename, NULL);
 		g_hash_table_insert (add_data->files_to_add,
 				     g_strdup (archive_pathname),
 				     add_file_new (file, archive_pathname));
-
-		g_free (archive_pathname);
-		g_free (basename);
 	}
 
 	_fr_archive_libarchive_save (archive,
@@ -2323,13 +2282,9 @@ fr_archive_libarchive_update_open_files (FrArchive           *archive,
 	{
 		GFile *temp_dir = G_FILE (scan_dir->data);
 		GFile *extracted_file = G_FILE (scan_file->data);
-		char  *archive_pathname;
-
-		archive_pathname = g_file_get_relative_path (temp_dir, extracted_file);
+		g_autofree char *archive_pathname = g_file_get_relative_path (temp_dir, extracted_file);
 		g_hash_table_insert (add_data->files_to_add, g_strdup (archive_pathname), add_file_new (extracted_file, archive_pathname));
 		add_data->n_files_to_add++;
-
-		g_free (archive_pathname);
 	}
 
 	_fr_archive_libarchive_save (archive,
@@ -2382,8 +2337,6 @@ fr_archive_libarchive_init (FrArchiveLibarchive *self)
 {
 	FrArchive *base = FR_ARCHIVE (self);
 
-	self->priv = fr_archive_libarchive_get_instance_private (self);
-
 	base->propAddCanReplace = TRUE;
 	base->propAddCanUpdate = TRUE;
 	base->propAddCanStoreFolders = TRUE;
